//
// Created by auric on 2/21/24.
//

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <nprint.h>
#include "sanity.h"
#include "config.h"
#include "debug.h"
#include "fex.h"
#include "install.h"

void help() {
    nprint("E", "Usage: fex (OPT) (PACKAGES)");
}

char* flagNames[] = {"INSTALL","LIST", "SEARCH", "FORCE", "UPGRADE"};

int main(int argc, char** argv) {
    unsigned int flags = 0;
    char* packages[argc - 1];
    int pkgCount = 0;
    int flagSource, flagUninstall = 0;

    if (argc < 2) {
        help();
        return 1;
    }

    for(int i = 1; i < argc; i++) {
        if(argv[i][0] == '-') {
            for(int j = 1; argv[i][j] != '\0'; j++) {
                switch(argv[i][j]) {
                    case 'i': // install, 0
                        flags |= FLAG_INSTALL;
                        break;
                    case 'S': // source, 1
                        //flags |= FLAG_SOURCE;
                        flagSource = 1;
                        break;
                    case 'r': // remove, 2
                        //flags |= FLAG_UNINSTALL;
                        flagUninstall = 1;
                        break;
                    case 'l': // list, 3
                        flags |= FLAG_LIST;
                        break;
                    case 's': // search, 4
                        flags |= FLAG_SEARCH;
                        break;
                    case 'f': // force, 5
                        flags |= FLAG_FORCE;
                        break;
                    case 'u': // upgrade, 6
                        flags |= FLAG_UPGRADE;
                        break;
                    default:
                        nprint("E", "Flag '%c' is unknown.", argv[i][j]);
                        return 1;
                }
            }
        } else {
            packages[pkgCount++] = argv[i];
        }
    }

    const int sysSanity = isSysSane();
    if ((isSyntaxSane(flags, pkgCount))) {
        return 1;
    }

#ifdef debugEnabled
        DEBUG_PRINT("sysSanity: %d", sysSanity);
        unsigned int flagValues[] = {0b00001, 0b00010, 0b00100, 0b01000, 0b10000};
        int numFlags = sizeof(flagValues) / sizeof(flagValues[0]);
        for(int i = 0; i < numFlags; i++) {
            if (flags & flagValues[i]) {
                nprint("D", "%s flag is set.", flagNames[i]);
            } else {
                nprint("D", "%s flag is not set.", flagNames[i]);
            }
        }
        for(int i = 0; i < pkgCount; i++) {
            nprint("D", "Package %d: %s", i, packages[i]);
        }
    #endif

    /* End Prelogic */
    DEBUG_PRINT("[!] Break parselogic");

    if(flags & FLAG_LIST) {
        DEBUG_PRINT("HIT: List");
        // list packages
    }
    if(flags & FLAG_SEARCH) {
        DEBUG_PRINT("HIT: Search");
        // search packages
    }
    if(!(getuid())) {
        if (flags & FLAG_INSTALL) {
            DEBUG_PRINT("HIT: Install. Source: %d; Uninstall: %d", flagSource, flagUninstall);
            installPackages(packages, pkgCount, flagSource, flagUninstall);
        }
        if(flags & FLAG_UPGRADE) {
            DEBUG_PRINT("HIT: Upgrade");
            //upgrade
        }
    } else {
        nprint("E", "Superuser permissions required for this operation.");
        return 1;
    }
    return 0;
}